package com.cyksj.service.mini;

import com.cyksj.model.entity.MiniPopularizeUserConversationChat;

/**
 * 项目名: yhlxj2
 * 文件名: MiniPopularizeCommonService
 * 创建者: JavaZou
 * 创建时间:2025/4/11 11:46
 */
public interface MiniPopularizeCommonService {

	void recoverUserLimit(Long userId, Boolean recoverNum);

	Boolean saveImgRecord(MiniPopularizeUserConversationChat record);

	void recordChatEndOfAnswer(String conversationId, Long userId, String content, MiniPopularizeUserConversationChat.Role role);

	void incrGptTimes(long userId, Integer oriNum, Integer addNum);

	Boolean checkImageIsCompleted(Long imgRecordId);
}
